"""
Module with persistent RawFrame classes.
"""

from datetime import datetime

from common.database.DataObject import DataObject
from common.database.DBMain import persistent

from micado.main.Simulator import SimulatorRun, SimBase


class BaseFrame(DataObject):
    """BaseFrame"""

    NAXIS1: int = persistent('Length of data in axis 1 [pixel]', int, -1)
    NAXIS2: int = persistent('Length of data in axis 2 [pixel]', int, -1)
    # imstat: Imstat = inline('Image statistics for the frame [None]', Imstat, None)


class RawFrame(BaseFrame, SimBase):
    """RawFrame"""

    simulator: SimulatorRun = persistent('The simulator producing this frame', SimulatorRun, None)

    # raw_fits_data: RawFitsData = persistent('The multi-extension raw image [None]', RawFitsData, None)
    # extension: int = persistent('The extension to extract [None]', int, 1)

    OBJECT: str = persistent('Object name [None]', str, '')
    OBSERVER: str = persistent('The observer [None]', str, '')
    DATE: datetime = persistent('UTC date the original data file was saved [None]', datetime, datetime(1990, 1, 1))
    DATE_OBS: datetime = persistent('UTC date at the start of the observation [None]', datetime, datetime(1990, 1, 1))
    MJD_OBS: float = persistent(
        'Modified Julian date at the start of the observation (JD-2400000.5) [day]', float, float('NaN'))
    LST: float = persistent(
        'Local sidereal time at the start of the observation expressed as the'
        ' number of seconds (a float) since UTC midnight [sec]',
        float, float('NaN'))
    UTC: float = persistent(
        'Universal Coordinated Time (UTC) at the start of the observation '
        'expressed as the number of seconds (a float) since UTC midnight [sec]',
        float, float('NaN'))

    # TODO enable ?
    chip: str = persistent('The CDD [None]', str, '')
    # instrument: Instrument = persistent('Information about the instrument [None]', Instrument, None)
    # chip: Chip = persistent('Information about the chip [None]', Chip, None)
    # observing_block: ObservingBlock = persistent('Information about the observing block [None]', ObservingBlock, None)
    # template: Template = persistent('Information about the template [None]', Template, None)

    # orientation = -1 # The orientation of the regions wrt the pixel coordinate system
    # PRSCX: int = persistent('Number of prescan pixels in the x-direction [pixel]', int, 0)
    # OVSCX: int = persistent('Number of overscan pixels in the x direction [pixel]', int, 0)
    # PRSCY: int = persistent('Number of prescan pixels in the y-direction [pixel]', int, 0)
    # OVSCY: int = persistent('Number of overscan pixels in the y direction [pixel]', int, 0)
    # PRSCXPRE: int = persistent(
    #  'Number of prescan pixels to skip in the x-direction at the edge of the chip [pixel]', int, 0)
    # OVSCXPRE: int = persistent(
    #  'Number of overscan pixels to skip in the x direction at the edge of the chip [pixel]', int, 0)
    # PRSCYPRE: int = persistent(
    #  'Number of prescan pixels to skip in the y-direction at the edge of the chip [pixel]', int, 0)
    # OVSCYPRE: int = persistent(
    #  'Number of overscan pixels to skip in the y direction at the edge of the chip [pixel]', int, 0)
    # PRSCXPST: int = persistent(
    #  'Number of prescan pixels to skip in the x-direction at the edge of the data region [pixel]', int, 0)
    # OVSCXPST: int = persistent(
    #  'Number of overscan pixels to skip in the x direction at the edge of the data region [pixel]', int, 0)
    # PRSCYPST: int = persistent(
    #  'Number of prescan pixels to skip in the y-direction at the edge of the data region [pixel]', int, 0)
    # OVSCYPST: int = persistent(
    #  'Number of overscan pixels to skip in the y direction at the edge of the data region [pixel]', int, 0)
    # prescan_x_stat: Imstat = inline('Statistics of the prescan region in the x-direction [None]', Imstat, None)
    # prescan_y_stat: Imstat = inline('Statistics of the prescan region in the y-direction [None]', Imstat, None)
    # overscan_x_stat: Imstat = inline('Statistics of the overscan region in the x-direction [None]', Imstat, None)
    # overscan_y_stat: Imstat = inline('Statistics of the overscan region in the x-direction [None]', Imstat, None)

    def unique_filename(self, input_filename, extra=None):
        """Generate unique filename."""
        SimBase.unique_filename(self, input_filename, extra=[self.DATE_OBS.strftime('%Y%m%dT%H%M%S'), self.chip])


class RawScienceFrame(RawFrame):
    """RawScienceFrame"""
    # filter: Filter = persistent('Information about the filter [None]', Filter, None)
    # astrom: Astrom = inline('Information about the astrometry [None]', Astrom, None)
    EXPTIME: float = persistent('Total observation time [sec]', float, float('NaN'))
    AIRMSTRT: float = persistent('Airmass at start of observation [None]', float, float('NaN'))
    AIRMEND: float = persistent('Airmass at end of observation [None]', float, float('NaN'))
    # process_params: RawScienceFrameParameters = persistent(
    #  'Processing Parameters [None]', RawScienceFrameParameters, None)
    # seeing: float = persistent('The calculated seeing for this frame [arcsec]', float, 0.0) # XXX TODO DBEVO

    def init_from_header(self, header):
        """Initialize from the header items."""
        for name, value in header.items():
            if hasattr(self, name):
                setattr(self, name, value)
